@@ -5,7 +5,7 @@ use serde::Deserialize;
 use serde_yaml::Value;
 
 use crate::common::expr::{BoE, LoE};
-use crate::common::{Env, If, Permissions, Uses, custom_error};
+use crate::common::{DockerUses, Env, If, Permissions, Uses, custom_error};
 
 use super::{Concurrency, Defaults};
 
@@ -134,7 +134,7 @@ pub enum StepBody {
 
         /// An optional shell to run in. Defaults to the job or workflow's
         /// default shell.
-        shell: Option<String>,
+        shell: Option<LoE<String>>,
     },
 }
 
@@ -162,7 +162,8 @@ pub struct Matrix {
 pub enum Container {
     Name(String),
     Container {
-        image: String,
+        #[serde(deserialize_with = "crate::common::docker_uses")]
+        image: DockerUses,
         credentials: Option<DockerCredentials>,
         #[serde(default)]
         env: LoE<Env>,
