@@ -3,13 +3,13 @@ use std::sync::LazyLock;
 use github_actions_models::workflow::Trigger;
 use github_actions_models::workflow::event::{BareEvent, BranchFilters, OptionalBody};
 
-use crate::audit::{Audit, audit_meta};
+use crate::audit::{Audit, AuditError, audit_meta};
 use crate::config::Config;
 use crate::finding::location::{Locatable as _, Routable};
 use crate::finding::{Confidence, Finding, Fix, FixDisposition, Severity};
 use crate::models::StepCommon;
 use crate::models::coordinate::{ActionCoordinate, ControlExpr, ControlFieldType, Toggle, Usage};
-use crate::models::workflow::{JobExt as _, NormalJob, Step, Steps};
+use crate::models::workflow::{JobCommon as _, NormalJob, Step, Steps};
 use crate::state::AuditState;
 
 use indexmap::IndexMap;
@@ -20,6 +20,7 @@ use super::AuditLoadError;
 /// The list of know cache-aware actions
 /// In the future we can easily retrieve this list from the static API,
 /// since it should be easily serializable
+#[allow(clippy::unwrap_used)]
 static KNOWN_CACHE_AWARE_ACTIONS: LazyLock<Vec<ActionCoordinate>> = LazyLock::new(|| {
     vec![
         // https://github.com/actions/cache/blob/main/action.yml
@@ -203,6 +204,7 @@ static KNOWN_CACHE_AWARE_ACTIONS: LazyLock<Vec<ActionCoordinate>> = LazyLock::ne
 
 /// A list of well-know publisher actions
 /// In the future we can retrieve this list from the static API
+#[allow(clippy::unwrap_used)]
 static KNOWN_PUBLISHER_ACTIONS: LazyLock<Vec<ActionCoordinate>> = LazyLock::new(|| {
     vec![
         // Public packages and/or binary distribution channels
@@ -398,7 +400,7 @@ impl CachePoisoning {
         let (coord, cache_usage) = self.evaluate_cache_usage(step)?;
 
         let (yaml_key, annotation) = match cache_usage {
-            Usage::Always => ("uses", "caching always restored here"),
+            Usage::Always => ("uses", "cache always restored here"),
             Usage::DefaultActionBehaviour => ("uses", "cache enabled by default here"),
             Usage::DirectOptIn => ("with", "opt-in for caching here"),
             Usage::ConditionalOptIn => ("with", "opt-in for caching might happen here"),
@@ -442,10 +444,11 @@ impl CachePoisoning {
             finding_builder = finding_builder.fix(fix);
         }
 
-        finding_builder.build(step.workflow()).ok()
+        finding_builder.build(step).ok()
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for CachePoisoning {
     fn new(_state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -454,11 +457,11 @@ impl Audit for CachePoisoning {
         Ok(Self)
     }
 
-    fn audit_normal_job<'doc>(
+    async fn audit_normal_job<'doc>(
         &self,
         job: &NormalJob<'doc>,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
         let steps = job.steps();
         let trigger = &job.parent().on;
@@ -495,11 +498,14 @@ mod tests {
     /// 4. Executes the provided test closure with the findings
     macro_rules! test_workflow_audit {
         ($audit_type:ty, $filename:expr, $workflow_content:expr, $test_fn:expr) => {{
-            let key = InputKey::local("fakegroup".into(), $filename, None::<&str>).unwrap();
+            let key = InputKey::local("fakegroup".into(), $filename, None::<&str>);
             let workflow = Workflow::from_string($workflow_content.to_string(), key).unwrap();
             let audit_state = AuditState::default();
             let audit = <$audit_type>::new(&audit_state).unwrap();
-            let findings = audit.audit_workflow(&workflow, &Config::default()).unwrap();
+            let findings = audit
+                .audit_workflow(&workflow, &Config::default())
+                .await
+                .unwrap();
 
             $test_fn(findings)
         }};
@@ -523,8 +529,8 @@ mod tests {
         fixed_document.source().to_string()
     }
 
-    #[test]
-    fn test_cache_disable_fix_opt_out_boolean() {
+    #[tokio::test]
+    async fn test_cache_disable_fix_opt_out_boolean() {
         let workflow_content = r#"
 name: Test Workflow
 on: release
@@ -549,6 +555,7 @@ jobs:
             |findings: Vec<Finding>| {
                 let fixed_content = apply_fix_for_snapshot(workflow_content, findings);
                 insta::assert_snapshot!(fixed_content, @r"
+
                 name: Test Workflow
                 on: release
 
@@ -569,8 +576,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_cache_disable_fix_opt_in_boolean() {
+    #[tokio::test]
+    async fn test_cache_disable_fix_opt_in_boolean() {
         let workflow_content = r#"
 name: Test Workflow
 on: release
@@ -593,6 +600,7 @@ jobs:
             |findings: Vec<Finding>| {
                 let fixed_content = apply_fix_for_snapshot(workflow_content, findings);
                 insta::assert_snapshot!(fixed_content, @r"
+
                 name: Test Workflow
                 on: release
 
@@ -610,8 +618,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_cache_disable_fix_opt_in_string() {
+    #[tokio::test]
+    async fn test_cache_disable_fix_opt_in_string() {
         let workflow_content = r#"
 name: Test Workflow
 on: release
@@ -641,8 +649,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_cache_disable_fix_non_configurable() {
+    #[tokio::test]
+    async fn test_cache_disable_fix_non_configurable() {
         let workflow_content = r#"
 name: Test Workflow
 on: release
