@@ -1,6 +1,8 @@
 use github_actions_models::common::Uses;
+use subfeature::Subfeature;
 
 use super::{Audit, AuditLoadError, AuditState, audit_meta};
+use crate::audit::AuditError;
 use crate::config::{Config, UsesPolicy};
 use crate::finding::{Confidence, Finding, Persona, Severity};
 use crate::models::uses::RepositoryUsesPattern;
@@ -30,7 +32,7 @@ impl UnpinnedUses {
             Uses::Docker(_) => {
                 if uses.unpinned() {
                     Some((
-                        "action is not pinned to a tag, branch, or hash ref".into(),
+                        "image is not pinned to a tag, branch, or hash ref".into(),
                         Severity::Medium,
                         Persona::default(),
                     ))
@@ -90,7 +92,7 @@ impl UnpinnedUses {
         &self,
         step: &impl StepCommon<'doc>,
         config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
         let Some(uses) = step.uses() else {
@@ -107,6 +109,7 @@ impl UnpinnedUses {
                         step.location()
                             .primary()
                             .with_keys(["uses".into()])
+                            .subfeature(Subfeature::new(0, uses.raw()))
                             .annotated(annotation),
                     )
                     .build(step)?,
@@ -117,6 +120,7 @@ impl UnpinnedUses {
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for UnpinnedUses {
     fn new(_state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -125,19 +129,19 @@ impl Audit for UnpinnedUses {
         Ok(Self)
     }
 
-    fn audit_step<'doc>(
+    async fn audit_step<'doc>(
         &self,
         step: &Step<'doc>,
         config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         self.process_step(step, config)
     }
 
-    fn audit_composite_step<'a>(
+    async fn audit_composite_step<'a>(
         &self,
         step: &CompositeStep<'a>,
         config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'a>>> {
+    ) -> Result<Vec<Finding<'a>>, AuditError> {
         self.process_step(step, config)
     }
 }
