@@ -21,7 +21,12 @@ use crate::{
     registry::input::RepoSlug,
 };
 
-const CONFIG_CANDIDATES: &[&str] = &[".github/zizmor.yml", "zizmor.yml"];
+const CONFIG_CANDIDATES: &[&str] = &[
+    ".github/zizmor.yml",
+    ".github/zizmor.yaml",
+    "zizmor.yml",
+    "zizmor.yaml",
+];
 
 #[derive(Error, Debug)]
 #[error("configuration error in {path}")]
@@ -254,7 +259,7 @@ impl UnpinnedUsesPolicies {
         &self,
         uses: &RepositoryUses,
     ) -> (Option<&RepositoryUsesPattern>, UsesPolicy) {
-        match self.policy_tree.get(&uses.owner) {
+        match self.policy_tree.get(uses.owner()) {
             Some(policies) => {
                 // Policies are ordered by specificity, so we can
                 // iterate and return eagerly.
