@@ -34,7 +34,7 @@ pub(crate) enum CollectionError {
     /// The input couldn't be converted into the expected model.
     /// This typically indicates a bug in `github-actions-models`.
     #[error("couldn't turn input into a an appropriate model")]
-    Model(#[source] anyhow::Error),
+    Model(#[from] serde_yaml::Error),
 
     /// The input couldn't be loaded into an internal yamlpath document.
     /// This typically indicates a bug in `yamlpath`.
@@ -72,6 +72,12 @@ pub(crate) enum CollectionError {
     #[error("invalid input: must have .yml or .yaml extension")]
     InvalidExtension,
 
+    /// Workflow-specific collection was requested, but the remote
+    /// input doesn't contain any workflows. This typically means the remote
+    /// repository doesn't have a `.github` or `.github/workflows` directory.
+    #[error("input {1} doesn't contain any workflows")]
+    RemoteWithoutWorkflows(#[source] ClientError, String),
+
     /// A GitHub API error occurred while fetching a remote input.
     #[error("GitHub API error while fetching remote input")]
     Client(#[from] ClientError),
@@ -80,10 +86,6 @@ pub(crate) enum CollectionError {
     #[error("I/O error: {0}")]
     Io(#[from] std::io::Error),
 
-    /// The input's name is missing.
-    #[error("invalid input: no filename component")]
-    MissingName,
-
     /// The input path isn't valid UTF-8.
     #[error("invalid path (not UTF-8): {1:?}")]
     InvalidPath(#[source] camino::FromPathError, PathBuf),
@@ -225,33 +227,20 @@ impl std::fmt::Display for InputKey {
 }
 
 impl InputKey {
-    pub(crate) fn local<P: AsRef<Utf8Path>>(
-        group: Group,
-        path: P,
-        prefix: Option<P>,
-    ) -> Result<Self, CollectionError> {
-        // All keys must have a filename component.
-        if path.as_ref().file_name().is_none() {
-            return Err(CollectionError::MissingName);
-        }
-
-        Ok(Self::Local(LocalKey {
+    pub(crate) fn local<P: AsRef<Utf8Path>>(group: Group, path: P, prefix: Option<P>) -> Self {
+        Self::Local(LocalKey {
             group,
             prefix: prefix.map(|p| p.as_ref().to_path_buf()),
             given_path: path.as_ref().to_path_buf(),
-        }))
+        })
     }
 
-    pub(crate) fn remote(slug: &RepoSlug, path: String) -> Result<Self, CollectionError> {
-        if Utf8Path::new(&path).file_name().is_none() {
-            return Err(CollectionError::MissingName);
-        }
-
-        Ok(Self::Remote(RemoteKey {
+    pub(crate) fn remote(slug: &RepoSlug, path: String) -> Self {
+        Self::Remote(RemoteKey {
             group: slug.into(),
             slug: slug.clone(),
             path: path.into(),
-        }))
+        })
     }
 
     /// Returns a path for this [`InputKey`] that's suitable for SARIF
@@ -296,8 +285,14 @@ impl InputKey {
         // NOTE: Safe unwraps, since the presence of a filename component
         // is a construction invariant of all `InputKey` variants.
         match self {
-            InputKey::Local(local) => local.given_path.file_name().unwrap(),
-            InputKey::Remote(remote) => remote.path.file_name().unwrap(),
+            InputKey::Local(local) => local
+                .given_path
+                .file_name()
+                .expect("expected input key to have a filename component"),
+            InputKey::Remote(remote) => remote
+                .path
+                .file_name()
+                .expect("expected input key to have a filename component"),
         }
     }
 
@@ -383,30 +378,40 @@ impl InputGroup {
         }
     }
 
-    fn collect_from_file(
+    async fn collect_from_file(
         path: &Utf8Path,
         options: &CollectionOptions,
     ) -> Result<Self, CollectionError> {
-        let config = Config::discover(options, || Config::discover_local(path))?;
+        let config = Config::discover(options, || Config::discover_local(path)).await?;
+
+        // Workflows can be named anything, including `dependabot.yml`
+        // (overlapping with Dependabot configs) and `action.yml` (overlapping
+        // with action definitions). Consequently, we make a best effort
+        // disambiguate them by looking at their parent path.
+        // See: https://github.com/zizmorcore/zizmor/issues/1341
+        let is_workflow_path = {
+            let resolved = path.canonicalize_utf8()?;
+
+            resolved
+                .parent()
+                .is_some_and(|parent| parent.ends_with(".github/workflows"))
+        };
 
         let mut group = Self::new(config);
 
         // When collecting individual files, we don't know which part
         // of the input path is the prefix.
         let (key, kind) = match (path.file_stem(), path.extension()) {
-            (Some("dependabot"), Some("yml" | "yaml")) => (
-                // NOTE: Safe unwrap because we just checked the filename.
-                InputKey::local(Group(path.as_str().into()), path, None).unwrap(),
+            (Some("dependabot"), Some("yml" | "yaml")) if !is_workflow_path => (
+                InputKey::local(Group(path.as_str().into()), path, None),
                 InputKind::Dependabot,
             ),
-            (Some("action"), Some("yml" | "yaml")) => (
-                // NOTE: Safe unwrap because we just checked the filename.
-                InputKey::local(Group(path.as_str().into()), path, None).unwrap(),
+            (Some("action"), Some("yml" | "yaml")) if !is_workflow_path => (
+                InputKey::local(Group(path.as_str().into()), path, None),
                 InputKind::Action,
             ),
             (Some(_), Some("yml" | "yaml")) => (
-                // NOTE: Safe unwrap because we just checked the filename.
-                InputKey::local(Group(path.as_str().into()), path, None).unwrap(),
+                InputKey::local(Group(path.as_str().into()), path, None),
                 InputKind::Workflow,
             ),
             _ => return Err(CollectionError::InvalidExtension),
@@ -420,11 +425,11 @@ impl InputGroup {
         Ok(group)
     }
 
-    fn collect_from_dir(
+    async fn collect_from_dir(
         path: &Utf8Path,
         options: &CollectionOptions,
     ) -> Result<Self, CollectionError> {
-        let config = Config::discover(options, || Config::discover_local(path))?;
+        let config = Config::discover(options, || Config::discover_local(path)).await?;
 
         let mut group = Self::new(config);
 
@@ -463,8 +468,7 @@ impl InputGroup {
                     .parent()
                     .is_some_and(|dir| dir.ends_with(".github/workflows"))
             {
-                // NOTE: Safe unwrap because we just checked the filename.
-                let key = InputKey::local(Group(path.as_str().into()), entry, Some(path)).unwrap();
+                let key = InputKey::local(Group(path.as_str().into()), entry, Some(path));
                 let contents = std::fs::read_to_string(entry).map_err(|e| {
                     CollectionError::Inner(
                         CollectionError::Io(e).into(),
@@ -479,8 +483,7 @@ impl InputGroup {
                 && entry.is_file()
                 && matches!(entry.file_name(), Some("action.yml" | "action.yaml"))
             {
-                // NOTE: Safe unwrap because we just checked the filename.
-                let key = InputKey::local(Group(path.as_str().into()), entry, Some(path)).unwrap();
+                let key = InputKey::local(Group(path.as_str().into()), entry, Some(path));
                 let contents = std::fs::read_to_string(entry).map_err(|e| {
                     CollectionError::Inner(
                         CollectionError::Io(e).into(),
@@ -498,8 +501,7 @@ impl InputGroup {
                     Some("dependabot.yml" | "dependabot.yaml")
                 )
             {
-                // NOTE: Safe unwrap because we just checked the filename.
-                let key = InputKey::local(Group(path.as_str().into()), entry, Some(path)).unwrap();
+                let key = InputKey::local(Group(path.as_str().into()), entry, Some(path));
                 let contents = std::fs::read_to_string(entry).map_err(|e| {
                     CollectionError::Inner(
                         CollectionError::Io(e).into(),
@@ -514,24 +516,26 @@ impl InputGroup {
         Ok(group)
     }
 
-    fn collect_from_repo_slug(
+    async fn collect_from_repo_slug(
         slug: RepoSlug,
         options: &CollectionOptions,
         gh_client: Option<&Client>,
     ) -> Result<Self, CollectionError> {
         let client = gh_client.ok_or_else(|| CollectionError::NoGitHubClient(slug.clone()))?;
 
-        let config = Config::discover(options, || Config::discover_remote(client, &slug))?;
+        let config = Config::discover(options, || Config::discover_remote(client, &slug)).await?;
         let mut group = Self::new(config);
 
         if options.mode_set.workflows_only() {
             // Performance: if we're *only* collecting workflows, then we
             // can save ourselves a full repo download and only fetch the
             // repo's workflow files.
-            client.fetch_workflows(&slug, options, &mut group)?;
+            client.fetch_workflows(&slug, options, &mut group).await?;
         } else {
             let before = group.len();
-            client.fetch_audit_inputs(&slug, options, &mut group)?;
+            client
+                .fetch_audit_inputs(&slug, options, &mut group)
+                .await?;
             let after = group.len();
             let len = after - before;
 
@@ -545,7 +549,7 @@ impl InputGroup {
         Ok(group)
     }
 
-    pub(crate) fn collect(
+    pub(crate) async fn collect(
         request: &str,
         options: &CollectionOptions,
         gh_client: Option<&Client>,
@@ -553,12 +557,12 @@ impl InputGroup {
         let path = Utf8Path::new(request);
 
         if path.is_file() {
-            Self::collect_from_file(path, options)
+            Self::collect_from_file(path, options).await
         } else if path.is_dir() {
-            Self::collect_from_dir(path, options)
+            Self::collect_from_dir(path, options).await
         } else {
             let slug = RepoSlug::from_str(request)?;
-            Self::collect_from_repo_slug(slug, options, gh_client)
+            Self::collect_from_repo_slug(slug, options, gh_client).await
         }
     }
 
@@ -588,7 +592,7 @@ impl InputRegistry {
         self.groups.values().map(|g| g.len()).sum()
     }
 
-    pub(crate) fn register_group(
+    pub(crate) async fn register_group(
         &mut self,
         name: &str,
         options: &CollectionOptions,
@@ -598,7 +602,7 @@ impl InputRegistry {
         // duplicated the input multiple times on the command line by accident.
         // We just ignore any duplicate registrations.
         if let btree_map::Entry::Vacant(e) = self.groups.entry(Group(name.into())) {
-            e.insert(InputGroup::collect(name, options, gh_client)?);
+            e.insert(InputGroup::collect(name, options, gh_client).await?);
         }
 
         Ok(())
@@ -635,12 +639,12 @@ mod tests {
 
     #[test]
     fn test_input_key_display() {
-        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", None).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", None);
         assert_eq!(local.to_string(), "file:///foo/bar/baz.yml");
 
         // No ref
         let slug = RepoSlug::from_str("foo/bar").unwrap();
-        let remote = InputKey::remote(&slug, ".github/workflows/baz.yml".into()).unwrap();
+        let remote = InputKey::remote(&slug, ".github/workflows/baz.yml".into());
         assert_eq!(
             remote.to_string(),
             "https://github.com/foo/bar/blob/HEAD/.github/workflows/baz.yml"
@@ -648,7 +652,7 @@ mod tests {
 
         // With a git ref
         let slug = RepoSlug::from_str("foo/bar@v1").unwrap();
-        let remote = InputKey::remote(&slug, ".github/workflows/baz.yml".into()).unwrap();
+        let remote = InputKey::remote(&slug, ".github/workflows/baz.yml".into());
         assert_eq!(
             remote.to_string(),
             "https://github.com/foo/bar/blob/v1/.github/workflows/baz.yml"
@@ -657,22 +661,20 @@ mod tests {
 
     #[test]
     fn test_input_key_local_presentation_path() {
-        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", None).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", None);
         assert_eq!(local.presentation_path(), "/foo/bar/baz.yml");
 
-        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo")).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo"));
         assert_eq!(local.presentation_path(), "/foo/bar/baz.yml");
 
-        let local =
-            InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo/bar/")).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo/bar/"));
         assert_eq!(local.presentation_path(), "/foo/bar/baz.yml");
 
         let local = InputKey::local(
             "fakegroup".into(),
             "/home/runner/work/repo/repo/.github/workflows/baz.yml",
             Some("/home/runner/work/repo/repo"),
-        )
-        .unwrap();
+        );
         assert_eq!(
             local.presentation_path(),
             "/home/runner/work/repo/repo/.github/workflows/baz.yml"
@@ -681,26 +683,23 @@ mod tests {
 
     #[test]
     fn test_input_key_local_sarif_path() {
-        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", None).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", None);
         assert_eq!(local.sarif_path(), "/foo/bar/baz.yml");
 
-        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo")).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo"));
         assert_eq!(local.sarif_path(), "bar/baz.yml");
 
-        let local =
-            InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo/bar/")).unwrap();
+        let local = InputKey::local("fakegroup".into(), "/foo/bar/baz.yml", Some("/foo/bar/"));
         assert_eq!(local.sarif_path(), "baz.yml");
 
         let local = InputKey::local(
             "fakegroup".into(),
             "/home/runner/work/repo/repo/.github/workflows/baz.yml",
             Some("/home/runner/work/repo/repo"),
-        )
-        .unwrap();
+        );
         assert_eq!(local.sarif_path(), ".github/workflows/baz.yml");
 
-        let local =
-            InputKey::local("fakegroup".into(), "./.github/workflows/baz.yml", Some(".")).unwrap();
+        let local = InputKey::local("fakegroup".into(), "./.github/workflows/baz.yml", Some("."));
         assert_eq!(local.sarif_path(), ".github/workflows/baz.yml");
     }
 }
